@@ -14,6 +14,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <bonobo/Bonobo.h>
+#include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-canvas-item.h>
 #include <bonobo/bonobo-object.h>
@@ -192,6 +193,7 @@ gbi_update (GnomeCanvasItem *item, double *item_affine,
 
 	CORBA_exception_init (&ev);
 	prepare_state (item, &state);
+
 	cuta = Bonobo_Canvas_Component_update (
 		gbi->priv->object,
 		&state, affine, clip_path, item_flags,
@@ -200,6 +202,7 @@ gbi_update (GnomeCanvasItem *item, double *item_affine,
 
 	if (!BONOBO_EX (&ev)){
 		if (cuta->width > 0 && cuta->height > 0){
+
 			ArtUta *uta;
 
 			uta = uta_from_cuta (cuta);
@@ -239,6 +242,8 @@ static void
 gbi_realize (GnomeCanvasItem *item)
 {
 	Gbi *gbi = GBI (item);
+	Bonobo_Gdk_WindowId id;
+
 	CORBA_Environment ev;
 
 	if (getenv ("DEBUG_BI"))
@@ -250,16 +255,25 @@ gbi_realize (GnomeCanvasItem *item)
 		gbi->priv->realize_pending = 1;
 		return;
 	}
+
+        proxy_size_allocate (
+                item->canvas,
+                &(GTK_WIDGET (item->canvas)->allocation),
+                BONOBO_CANVAS_ITEM(item));
 		
 	g_signal_connect (item->canvas, "size_allocate",
 			  G_CALLBACK (proxy_size_allocate), item);
 
 	CORBA_exception_init (&ev);
 	gdk_flush ();
-	Bonobo_Canvas_Component_realize (
-		gbi->priv->object, 
-		GDK_WINDOW_XWINDOW (item->canvas->layout.bin_window),
-		&ev);
+
+	id = bonobo_control_window_id_from_x11
+		(GDK_WINDOW_XWINDOW (item->canvas->layout.bin_window));
+
+	Bonobo_Canvas_Component_realize (gbi->priv->object, id, &ev);
+
+	CORBA_free (id);
+
 	CORBA_exception_free (&ev);
 }
 
@@ -287,6 +301,7 @@ gbi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	Gbi *gbi = GBI (item);
 	CORBA_Environment ev;
 	Bonobo_Canvas_State state;
+	Bonobo_Gdk_WindowId id;
 	
 	if (getenv ("DEBUG_BI"))
 		g_message ("draw: %d %d %d %d", x, y, width, height);
@@ -298,12 +313,16 @@ gbi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	CORBA_exception_init (&ev);
 
 	prepare_state (item, &state);
+	id = bonobo_control_window_id_from_x11
+		(GDK_WINDOW_XWINDOW (drawable));
+	
 	Bonobo_Canvas_Component_draw (
 		gbi->priv->object,
 		&state,
-		GDK_WINDOW_XWINDOW (drawable),
-		x, y, width, height,
+		id, x, y, width, height,
 		&ev);
+
+	CORBA_free (id);
 	CORBA_exception_free (&ev);
 }
 
@@ -447,7 +466,7 @@ gdk_event_to_bonobo_event (GdkEvent *event)
 		e->_d = Bonobo_Gdk_MOTION;
 		e->_u.motion.time = event->motion.time;
 		e->_u.motion.x = event->motion.x;
-		e->_u.motion.y = event->motion.x;
+		e->_u.motion.y = event->motion.y;
 		e->_u.motion.x_root = event->motion.x_root;
 		e->_u.motion.y_root = event->motion.y_root;
 #ifdef FIXME
@@ -491,6 +510,7 @@ gdk_event_to_bonobo_event (GdkEvent *event)
 		e->_u.crossing.y = event->crossing.y;
 		e->_u.crossing.x_root = event->crossing.x_root;
 		e->_u.crossing.y_root = event->crossing.y_root;
+		e->_u.crossing.state = event->crossing.state;
 
 		switch (event->crossing.mode){
 		case GDK_CROSSING_NORMAL:
@@ -556,7 +576,11 @@ gbi_set_property (GObject      *object,
 
 		gbi->priv->object = bonobo_object_release_unref (gbi->priv->object, &ev);
 
-		factory = bonobo_value_get_corba_object (value);
+                /* FIXME: I can't make it work as corba_object. 
+                   Using bonobo_unknown for now. */
+		/* factory = bonobo_value_get_corba_object (value); */
+               
+		factory = bonobo_value_get_unknown (value);
 
 		g_return_if_fail (factory != CORBA_OBJECT_NIL);
 
@@ -580,12 +604,6 @@ gbi_set_property (GObject      *object,
 			return;
 		}
 
-		/* Initial size notification */
-		proxy_size_allocate (
-			GNOME_CANVAS_ITEM (gbi)->canvas,
-			&(GTK_WIDGET (GNOME_CANVAS_ITEM (gbi)->canvas)->allocation),
-			gbi);
-	
 		if (gbi->priv->realize_pending){
 			gbi->priv->realize_pending = 0;
 			gbi_realize (GNOME_CANVAS_ITEM (gbi));
@@ -639,6 +657,8 @@ bonobo_canvas_item_class_init (BonoboCanvasItemClass *object_class)
 	GnomeCanvasItemClass *item_class =
 		(GnomeCanvasItemClass *) object_class;
 
+	gobject_class->set_property = gbi_set_property;
+
 	g_object_class_install_property (
 		gobject_class,
 		PROP_CORBA_FACTORY,
@@ -660,7 +680,6 @@ bonobo_canvas_item_class_init (BonoboCanvasItemClass *object_class)
 			G_PARAM_WRITABLE));
 	
 	gobject_class->finalize     = gbi_finalize;
-	gobject_class->set_property = gbi_set_property;
 
 	item_class->update     = gbi_update;
 	item_class->realize    = gbi_realize;
@@ -672,13 +691,42 @@ bonobo_canvas_item_class_init (BonoboCanvasItemClass *object_class)
 	item_class->event      = gbi_event;
 }
 
+static gboolean
+gbi_idle_handler (GnomeCanvasItem *item)
+{
+	gnome_canvas_item_request_update (item);
+        return FALSE;
+}
+
 static void
 impl_Bonobo_Canvas_ComponentProxy_requestUpdate (PortableServer_Servant servant,
 					         CORBA_Environment     *ev)
 {
 	ComponentProxyServant *item_proxy = (ComponentProxyServant *) servant;
 
-	gnome_canvas_item_request_update (item_proxy->item_bound);
+        if (item_proxy->item_bound->canvas->idle_id == 0)
+        {
+        	gnome_canvas_item_request_update (item_proxy->item_bound);
+        }
+        else
+        {
+                /* Problem: It is possible to get here at a time when the canvas
+                 * is in the middle of a do_update.  This happens because
+                 * this proxy call might be waiting in the queue when
+                 * the this object calls Bonobo_Canvas_Component_update.
+                 *
+                 * Solution:
+                 * The canvas is either in its do_update routine or it is
+                 * waiting to do an update.  If it is waiting this idle_handler
+                 * will get called just before the canvas's handler.  If it is
+                 * in the middle of a do_update, then this will queue for the
+                 * next one.
+                 */
+
+                 g_idle_add_full(GDK_PRIORITY_REDRAW-6,
+                                 (GSourceFunc)gbi_idle_handler, 
+                                 item_proxy->item_bound, NULL);
+        }
 
 }
 					    
@@ -692,7 +740,9 @@ impl_Bonobo_Canvas_ComponentProxy_grabFocus (PortableServer_Servant servant,
 	ComponentProxyServant *item_proxy = (ComponentProxyServant *) servant;
 	GdkCursor *cursor;
 
-	cursor = gdk_cursor_new ((GdkCursorType) cursor_type);
+	cursor = gdk_cursor_new_for_display
+		(gtk_widget_get_display (GTK_WIDGET (item_proxy->item_bound->canvas)),
+		 (GdkCursorType) cursor_type);
 
 	gnome_canvas_item_grab (item_proxy->item_bound, mask, cursor, time);
 }
@@ -719,10 +769,12 @@ impl_Bonobo_Canvas_ComponentProxy_getUIContainer (PortableServer_Servant servant
 	return bonobo_object_dup_ref (item_proxy->ui_container, NULL);
 }
 
+static PortableServer_ServantBase__epv item_proxy_base_epv;
+
 static POA_Bonobo_Canvas_ComponentProxy__epv item_proxy_epv;
 
 static POA_Bonobo_Canvas_ComponentProxy__vepv item_proxy_vepv = {
-	NULL,
+	&item_proxy_base_epv,
 	&item_proxy_epv
 };
 
@@ -736,6 +788,8 @@ create_proxy (GnomeCanvasItem *item)
 	ComponentProxyServant *item_proxy = g_new0 (ComponentProxyServant, 1);
 	CORBA_Environment ev;
 	
+        item_proxy->proxy_servant.vepv = &item_proxy_vepv;
+
 	CORBA_exception_init (&ev);
 	POA_Bonobo_Canvas_ComponentProxy__init ((PortableServer_Servant) item_proxy, &ev);
 
@@ -744,7 +798,6 @@ create_proxy (GnomeCanvasItem *item)
 	item_proxy_epv.ungrabFocus    = impl_Bonobo_Canvas_ComponentProxy_ungrabFocus;
 	item_proxy_epv.getUIContainer = impl_Bonobo_Canvas_ComponentProxy_getUIContainer;
 
-	item_proxy->proxy_servant.vepv = &item_proxy_vepv;
 	item_proxy->item_bound = item;
 
 	item_proxy->oid = PortableServer_POA_activate_object (
